/*
 * The Kibitz Library
 *
 * (c) 2005-2012 Tomas Valla <tom@ucw.cz>
 *
 * This software may be freely distributed and used according to the terms
 * of the GNU Lesser General Public License. See COPYING for details.
 */

#ifndef _KIBITZ_KIBITZ_H_
#define _KIBITZ_KIBITZ_H_

#include <ucw/clists.h>
#include <ucw/mainloop.h>
#include <custom/custom.h>

uns kibitz_make_index(struct fastbuf *input, struct fastbuf *output);
uns kibitz_make_index_file(byte *inputfile, byte *indexname);

void kibitz_init(byte *cfg_file);
void kibitz_cleanup(void);

void kibitz_open_index(byte *idxname);
void kibitz_close_index(void);


struct kibitz_query {
	int error_code;
	byte *error_msg;

	clist results;
	uns num_results;
	uns max_results;

	/* Internal stuff, do not touch */
	clist expr;
	uns num_exprs;
	struct mempool *pool;
};

struct kibitz_expr {
	cnode node;
	byte *word;
	uns wlen;
	double bonus_factor;
};

enum kibitz_error_code {
	KIBITZ_ERR_OK = 0,

	/* FIXME */
};

#include <kibitz/index.h>

struct kibitz_result {
	cnode node;
	int quality;
	struct phrase phrase;
};

struct kibitz_query *kibitz_new_query(void);
void kibitz_free_query(struct kibitz_query *q);
void kibitz_dump_query(struct kibitz_query *q);
int kibitz_add_expr(struct kibitz_query *q, byte *word, uns wlen, double bonus_factor);

int kibitz_init_query(struct kibitz_query *q);
int kibitz_get_results(struct kibitz_query *q);

void kibitz_httpd_init(byte *idx_name, byte *cfg_name);
void kibitz_httpd_exec(void);

#define MAX_LINE_LEN	1024

// FIXME: it would be better to move required fields to kibiz_query and keep this structure private to httpd.c
struct connection {
	cnode node;
	struct main_rec_io rio;
	struct mempool *mp;

	int status;
	int http_subver;
	int get_method;
	int ignore_input;
	int timer_active;
	int debug_mode;
	int output_format;
	int result_cnt;
	int replied;

	byte *arg_buf;
	byte *index_name;
	byte *input_enc;
	byte *output_enc;
	byte *reply_buf;
	byte *qw_buf;
	byte *ip_addr;

	clist query_words;
	struct kibitz_query *q;
};

#endif
